from cudametro import construct as cst
import numpy as np
import matplotlib.pyplot as plt
import tqdm as tqdm

# =============================================================================
# Edit the following lines to change the parameters of the simulation
# =============================================================================
fname = "fig1_configs"          # Configuration file [options p1.json, p2.json, p3.json, p4.json]
in_dir = "input_parameters/"                             # Input directory
# =============================================================================


# Universal parameters
save_dir_flag = False
T_arr = None
spin = 1.0
mat = None
size = 32

for i in range(4):
    fs_name = f"fig1_configs/p{i+1}.json"
    print(f"Running {fs_name}")
    test_mc0 = cst.MonteCarlo(config=fs_name, input_folder=in_dir)
    test_mc0.mc_init()
    test_mc0.display_material()
    M, X = np.array([]), np.array([])
    for t in test_mc0.T:
        test_mc0.grid_reset()
        test_mc0.generate_random_numbers(test_mc0.S_Wrap)
        m, x = test_mc0.run_mc_tc_3636(t)
        M = np.append(M, m)
        X = np.append(X, x)
    if not save_dir_flag:
        unified_save_dir = test_mc0.save_directory
        save_dir_flag = True
        save_dir = test_mc0.save_directory
        T_arr = test_mc0.T
        spin = test_mc0.spin
        mat = test_mc0.MAT_NAME
        size = test_mc0.size
    np.save(f"{unified_save_dir}/BQM_{test_mc0.size}_{test_mc0.Blocks}", M)
    np.save(f"{unified_save_dir}/BQX_{test_mc0.size}_{test_mc0.Blocks}", X)


    X[0] = 0.0

    plt.plot(T_arr, M/test_mc0.spin, label="M")
    plt.title(f"Magnetization vs Temperature of {mat} at size {test_mc0.size}x{test_mc0.size}")
    plt.savefig(f"{unified_save_dir}/M_{test_mc0.size}.png")
    plt.close()
    plt.plot(T_arr, X/X.max(), label="X")
    plt.title(f"Susceptibility vs Temperature of {mat} at size {test_mc0.size}x{test_mc0.size}")
    plt.savefig(f"{unified_save_dir}/X_{test_mc0.size}.png")
    plt.close()
    test_mc0 = None

M_1, X_1 = np.load(f"{unified_save_dir}/BQM_32_128.npy"), np.load(f"{unified_save_dir}/BQX_32_128.npy")
M_2, X_2 = np.load(f"{unified_save_dir}/BQM_32_256.npy"), np.load(f"{unified_save_dir}/BQX_32_256.npy")
M_3, X_3 = np.load(f"{unified_save_dir}/BQM_32_512.npy"), np.load(f"{unified_save_dir}/BQX_32_512.npy")
M_4, X_4 = np.load(f"{unified_save_dir}/BQM_32_1024.npy"), np.load(f"{unified_save_dir}/BQX_32_1024.npy")

Figure = plt.figure()
plt.plot(T_arr, M_1/spin, label="M_12.5")
plt.plot(T_arr, M_2/spin, label="M_25")
plt.plot(T_arr, M_3/spin, label="M_50")
plt.plot(T_arr, M_4/spin, label="M_100")
plt.title(f"Magnetization vs Temperature of {mat} at size {size}x{size}")
plt.legend()
plt.savefig(f"{unified_save_dir}/M_{size}_all.png")
plt.close()

Figure = plt.figure()
plt.plot(T_arr[1:], X_1[1:]/X_1.max(), label="X_12.5")
plt.plot(T_arr[1:], X_2[1:]/X_2.max(), label="X_25")
plt.plot(T_arr[1:], X_3[1:]/X_3.max(), label="X_50")
plt.plot(T_arr[1:], X_4[1:]/X_4.max(), label="X_100")
plt.title(f"Susceptibility vs Temperature of {mat} at size {size}x{size}")
plt.legend()
plt.savefig(f"{unified_save_dir}/X_{size}_all.png")
plt.close()


import alt_mat_1_construct as cst
import cudametro.montecarlo as mc
import numpy as np
import tqdm as tqdm
import matplotlib.pyplot as plt

mc2 = cst.alt_Montecarlo(config1="../configs/test_config.json", config2="../configs/test_config.json")
mc2.MAT_NAME = "MnCrI6"
mc2.mc_init(S1=1.50, S2=2.00)
print(mc2.T)
'''
for i in tqdm.tqdm(range(mc2.S_Wrap), desc="Stability Runs", unit="runs", colour="green"):
    mc2.generate_random_numbers(mc2.S_Wrap)
    np.save(f"{mc2.save_direcotry}/grid_{i:04d}", mc2.run_mc_3636(mc2.T[0]))
mc2.Analyze()
mc2.spin_view()
#mc2.quiver_view()
'''

M, X = np.array([]), np.array([])
for t in mc2.T:
    #mc2.grid_reset()
    mc2.generate_random_numbers(mc2.S_Wrap)
    m, x = mc2.run_mc_tc_3636(t)
    M = np.append(M, m)
    X = np.append(X, x)

np.save(f"{mc2.save_direcotry}/M_{mc2.size}", M)
np.save(f"{mc2.save_direcotry}/X_{mc2.size}", X)

plt.plot(mc2.T, M/mc2.spin, label="M")
plt.plot(mc2.T, X/X.max(), label="X")
plt.title(f"Magnetization vs Temperature of {mc2.MAT_NAME} at size {mc2.size}x{mc2.size}")
plt.savefig(f"{mc2.save_direcotry}/M_{mc2.size}.png")


import construct as cst
import cudametro.montecarlo as mc
import numpy as np
import tqdm as tqdm
import sys
import os

test_mc0 = cst.MonteCarlo(config="../../configs/test_config.json")
test_mc0.mc_init()

args = sys.argv
if len(args) != 2:
    print("Usage: python3 test_main.py <config_file>")
    exit(1)
folder = args[1]

viewer = cst.Analyze(folder, reverse=True)
viewer.spin_view()
viewer.quiver_view()


# Description: This script is used to visualize the results of the simulation. It takes the folder containing the results as input and generates the following plots:
# 1. Spin configuration at the end of the simulation
# 2. Quiver plot of the spins
#--------------------------------------------

import construct as cst
import sys
import os

args = sys.argv
if len(args) != 2:
    print("Usage: python3 test_main.py <config_file>")
    exit(1)
folder = args[1]

viewer = cst.Analyze(folder, reverse=False)

# Visualize the results
# Spin configuration at the end of the simulation
viewer.spin_view()
# Quiver plot of the spins
viewer.quiver_view()



# Testing script for the CUDAMETRO package
# Currently in version 1.2.0


# Importing the necessary packages

print("=== Testing the CUDAMETRO package ===")
print("=== Testing the import of the necessary packages ===")
print(" Searching for CUDAMETRO")
try:
    import cudametro as cm
except ImportError:
    print("Please install the package using the following command: pip install cudametro")

if "cm" in globals():
    print(f"CUDA METRO package is installed. Version: {cm.__version__}")

print(" Searching for numpy")
try:
    import numpy as np
except ImportError:
    print("Please install the package using the following command: pip install numpy")
if "np" in globals():
    print(f"NumPy package is installed. Version: {np.__version__}")

print(" Searching for matplotlib")
try:
    import matplotlib as plt
except ImportError:
    print("Please install the package using the following command: pip install matplotlib")
if "plt" in globals():
    print(f"Matplotlib package is installed. Version: {plt.__version__}")

print(" Searching for tqdm")
try:
    import tqdm as tqdm
except ImportError:
    print("Please install the package using the following command: pip install tqdm")
if "tqdm" in globals():
    print(f"TQDM package is installed. Version: {tqdm.__version__}")

print(" Searching for seaborn")
try:
    import seaborn
except ImportError:
    print("Please install the package using the following command: pip install seaborn")
if "seaborn" in globals():
    print(f"Seaborn package is installed. Version: {seaborn.__version__}")

print(" Searching for pyCUDA")    
try:
    import pycuda
except ImportError:
    print("Please install the package using the following command: pip install pycuda")
    print("Please refer to the following link for installation instructions: https://wiki.tiker.net/PyCuda/Installation/")
if "pycuda" in globals():
    print(f"PyCUDA package is installed. Version: {pycuda.VERSION_TEXT}")

print("=== All packages are installed ===")

# Testing GPU availability
print("=== Testing GPU availability ===")
print(" Searching for GPU")
try:
    import pycuda.driver as cuda
    cuda.init()
    print(f"GPU found: {cuda.Device.count()} devices found")
    for i in range(cuda.Device.count()):
        dev = cuda.Device(i)
        print(f"Device {i}: {dev.name()}")
except ImportError:
    print("Please install the package using the following command: pip install pycuda")
    print("Please refer to the following link for installation instructions: https://wiki.tiker.net/PyCuda/Installation/")

# Print GPU information
print("=== GPU Information ===")
print('\x1b[6;30;42m')

for i in range(cuda.Device.count()):
    print(f"Device {i}: {cuda.Device(i).name()}")
    print(f"Compute Capability: {cuda.Device(i).compute_capability()}")
    print(f"Total Memory: {cuda.Device(i).total_memory()/1024**2} MB")
    print(f"Max Threads per Block: {cuda.Device(i).max_threads_per_block}")
    print(f"Max Threads per Multiprocessor: {cuda.Device(i).max_threads_per_multiprocessor}")
    print(f"Max Grid Size: {cuda.Device(i).max_grid_dim_x}, {cuda.Device(i).max_grid_dim_y}, {cuda.Device(i).max_grid_dim_z}")
    print(f"Max Threads per Block: {cuda.Device(i).max_threads_per_block}")

print('\x1b[0m')

if cuda.Device(i).compute_capability() < (5,0):
    print("Please ensure that the GPU has a compute capability of at least 5.0")
    print("Please refer to the following link for further information: https://developer.nvidia.com/cuda-gpus")

print("CHECK COMPLETE")
print("=== All tests are complete ===")
print("=== Please refer to the README.md file for further instructions ===")
print("=== If you encounter any issues, please refer to the documentation ===")
print("=== For any further queries, please contact the author ===")
print("=== Thank you for using the CUDAMETRO package ===")


# Main file to run the Monte Carlo simulation 
#--------------------------------------------

from cudametro import construct as cst
import numpy as np
import tqdm as tqdm

# Set your config file here
test_mc0 = cst.MonteCarlo(config="sample_config.json")

# Initialize the Monte Carlo simulation
test_mc0.mc_init()
test_mc0.display_material()
for i in tqdm.tqdm(range(test_mc0.S_Wrap), desc="Stability Runs", unit="runs", colour="green"):
    test_mc0.generate_random_numbers(test_mc0.stability_runs)
    np.save(f"{test_mc0.save_directory}/grid_{i:04d}", test_mc0.run_mc_dmi_66612(test_mc0.T[0]))

# Save the final state of the simulation

print("Simulation Completed")
print(f"Final state saved in {test_mc0.save_directory}")
# To visualize the final state, use the "visualize.py" script with appropriate directory
print("To visualize the final state, use the visualize.py script with appropriate directory")
print(f"Example: python visualize.py {test_mc0.save_directory}")
inp = input("Press Y/N to continue to analysis mode: ")
#--------------------------------------------

if inp == "Y" or "y" or "yes" or "Yes":
    viewer = cst.Analyze(test_mc0.save_directory, reverse=False)
    viewer.spin_view()

#--------------------------------------------

if inp == "N" or "n" or "no" or "No":
    print("Exiting")
    exit(0)


import numpy as np
import cupy as cp
import json
import os
import sys
sys.path.append("../utilities/")
import Material_Reader as rm
import datetime
import tqdm as tqdm

kb = 8.617333262145e-2 # meV/K

# Definitions of neighbours
def n1_6_6_6_12(pt, size):
    nbs = np.zeros(6).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-1)*size + col-1 + size*size)%(size*size)
    nbs[1] = (row*size     + col+1 + size*size)%(size*size)
    nbs[2] = ((row-1)*size + col   + size*size)%(size*size)
    nbs[3] = ((row)*size   + col-1 + size*size)%(size*size)
    nbs[4] = ((row+1)*size + col+1 + size*size)%(size*size)
    nbs[5] = ((row+1)*size + col   + size*size)%(size*size)
    return nbs

def n2_6_6_6_12(pt, size):
    nbs = np.zeros(6).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-2)*size + col-1 + size*size)%(size*size)
    nbs[1] = ((row-1)*size + col+1 + size*size)%(size*size)
    nbs[2] = ((row+1)*size + col-1 + size*size)%(size*size)
    nbs[3] = ((row-1)*size + col-2 + size*size)%(size*size)
    nbs[4] = ((row+1)*size + col+1 + size*size)%(size*size)
    nbs[5] = ((row+1)*size + col+2 + size*size)%(size*size)
    return nbs

def n3_6_6_6_12(pt, size):
    nbs = np.zeros(6).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-2)*size + col-2 + size*size)%(size*size)
    nbs[1] = ((row)*size   + col+2 + size*size)%(size*size)
    nbs[2] = ((row-2)*size + col   + size*size)%(size*size)
    nbs[3] = ((row)*size   + col-2 + size*size)%(size*size)
    nbs[4] = ((row+2)*size + col+2 + size*size)%(size*size)
    nbs[5] = ((row+2)*size + col   + size*size)%(size*size)
    return nbs

def n4_6_6_6_12(pt, size):
    nbs = np.zeros(12).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-2)*size + col-3 + size*size)%(size*size)
    nbs[1] = ((row-3)*size + col-1 + size*size)%(size*size)
    nbs[2] = ((row-3)*size + col-2 + size*size)%(size*size)
    nbs[3] = ((row+1)*size + col+3 + size*size)%(size*size)
    nbs[4] = ((row-1)*size + col+2 + size*size)%(size*size)
    nbs[5] = ((row-2)*size + col+1 + size*size)%(size*size)
    nbs[6] = ((row+2)*size + col-1 + size*size)%(size*size)
    nbs[7] = ((row+1)*size + col-2 + size*size)%(size*size)
    nbs[8] = ((row-1)*size + col-3 + size*size)%(size*size)
    nbs[9] = ((row+3)*size + col+2 + size*size)%(size*size)
    nbs[10] = ((row+1)*size+ col+3 + size*size)%(size*size)
    nbs[11] = ((row+2)*size+ col+3 + size*size)%(size*size)
    return nbs

def n1_4_4_4_8(pt, size):
    nbs = np.zeros(4).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-1)*size + col-1 + size*size)%(size*size)
    nbs[1] = (row*size     + col+1 + size*size)%(size*size)
    nbs[2] = ((row-1)*size + col   + size*size)%(size*size)
    nbs[3] = ((row)*size   + col-1 + size*size)%(size*size)
    return nbs

def n2_4_4_4_8(pt, size):
    nbs = np.zeros(4).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-1)*size + col-1 + size*size)%(size*size)
    nbs[1] = ((row-1)*size + col+1 + size*size)%(size*size)
    nbs[2] = ((row+1)*size + col-1 + size*size)%(size*size)
    nbs[3] = ((row+1)*size + col+1 + size*size)%(size*size)
    return nbs

def n3_4_4_4_8(pt, size):
    nbs = np.zeros(4).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-2)*size + col   + size*size)%(size*size)
    nbs[1] = ((row  )*size + col-2 + size*size)%(size*size)
    nbs[2] = ((row+2)*size + col   + size*size)%(size*size)
    nbs[3] = ((row  )*size + col+2 + size*size)%(size*size)
    return nbs

def n4_4_4_4_8(pt, size):
    nbs = np.zeros(8).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-2)*size + col-1 + size*size)%(size*size)
    nbs[1] = ((row-1)*size + col-2 + size*size)%(size*size)
    nbs[2] = ((row-2)*size + col+1 + size*size)%(size*size)
    nbs[3] = ((row-1)*size + col+2 + size*size)%(size*size)
    nbs[4] = ((row+1)*size + col-2 + size*size)%(size*size)
    nbs[5] = ((row+2)*size + col-1 + size*size)%(size*size)
    nbs[6] = ((row+1)*size + col+2 + size*size)%(size*size)
    nbs[7] = ((row+2)*size + col+1 + size*size)%(size*size)
    return nbs

def n1_2_2_4_2(pt, size):
    nbs = np.zeros(2).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-1)*size + col   + size*size)%(size*size)
    nbs[1] = ((row+1)*size + col   + size*size)%(size*size)
    return nbs

def n2_2_2_4_2(pt, size):
    nbs = np.zeros(2).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row+1)*size + col-1 + size*size)%(size*size)
    nbs[1] = ((row+1)*size + col+1 + size*size)%(size*size)
    return nbs

def n3_2_2_4_2(pt, size):
    nbs = np.zeros(4).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row+1)*size + (col-1+size)%size + size*size)%(size*size)
    nbs[1] = ((row+1)*size + col+1 + size*size)%(size*size)
    nbs[2] = (size - (row-1)*size + col-1 + size*size)%(size*size)
    nbs[3] = ((row-1)*size + col+1 + size*size)%(size*size)
    return nbs

def n4_2_2_4_2(pt, size):
    nbs = np.zeros(2).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-2)*size + col   + size*size)%(size*size)
    nbs[1] = ((row-2)*size + col   + size*size)%(size*size)
    return nbs

def n1_2_4_2_4(pt, size):
    nbs = np.zeros(2).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = (row*size + col-1   + size*size)%(size*size)
    nbs[1] = (row*size + col+1   + size*size)%(size*size)
    return nbs

def n2_2_4_2_4(pt, size):
    nbs = np.zeros(4).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-1)*size + col   + size*size)%(size*size)
    nbs[1] = ((row+1)*size + col   + size*size)%(size*size)
    nbs[2] = ((row-1)*size + col-1 + size*size)%(size*size)
    nbs[3] = ((row+1)*size + col-1 + size*size)%(size*size)
    return nbs

def n3_2_4_2_4(pt, size):
    nbs = np.zeros(2).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-2)*size + col   + size*size)%(size*size)
    nbs[1] = ((row+2)*size + col   + size*size)%(size*size)
    return nbs

def n4_2_4_2_4(pt, size):
    nbs = np.zeros(4).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-2)*size + col-1   + size*size)%(size*size)
    nbs[1] = ((row+2)*size + col+1   + size*size)%(size*size)
    nbs[2] = ((row-2)*size + col-1   + size*size)%(size*size)
    nbs[3] = ((row+2)*size + col+1   + size*size)%(size*size)
    return nbs

def n1_3_6_3_6(pt, size):
    nbs = np.zeros(3).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row+1)*size + col+0 + size*size)%(size*size)
    nbs[1] = ((row-1)*size + col+1 + size*size)%(size*size)
    nbs[2] = ((row-0)*size + col-1 + size*size)%(size*size)
    return nbs

def n2_3_6_3_6(pt, size):
    nbs = np.zeros(6).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row+1)*size + col+1 + size*size)%(size*size)
    nbs[1] = ((row-1)*size + col+2 + size*size)%(size*size)
    nbs[2] = ((row-2)*size + col+1 + size*size)%(size*size)
    nbs[3] = ((row-1)*size + col-1 + size*size)%(size*size)
    nbs[4] = ((row+1)*size + col-2 + size*size)%(size*size)
    nbs[5] = ((row+2)*size + col-1 + size*size)%(size*size)
    return nbs

def n3_3_6_3_6(pt, size):
    nbs = np.zeros(3).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row+2)*size + col   + size*size)%(size*size)
    nbs[1] = ((row-2)*size + col+2 + size*size)%(size*size)
    nbs[2] = ((row  )*size + col-2 + size*size)%(size*size)
    return nbs

def n4_3_6_3_6(pt, size):
    nbs = np.zeros(6).astype(np.int32)
    row, col = pt//size, pt%size
    nbs[0] = ((row-1)*size + col+3 + size*size)%(size*size)
    nbs[1] = ((row-3)*size + col+1 + size*size)%(size*size)
    nbs[2] = ((row-2)*size + col-1 + size*size)%(size*size)
    nbs[3] = ((row+2)*size + col-3 + size*size)%(size*size)
    nbs[4] = ((row-3)*size + col+2 + size*size)%(size*size)
    nbs[5] = ((row+1)*size + col+2 + size*size)%(size*size)
    return nbs

# Definitions of Hamiltonians

def H_6_6_6_12_DM1(mat, grid, pt, sx, sy, sz, dmi, B, size):
    H = 0.0
    d_plane = mat[25]
    n1 = n1_6_6_6_12(pt, size)
    n2 = n2_6_6_6_12(pt, size)
    n3 = n3_6_6_6_12(pt, size)
    n4 = n4_6_6_6_12(pt, size)
    
    for i in range(6):
        H += mat[1]*(sx*grid[n1[i]*3] + sy*grid[n1[i]*3+1] + sz*grid[n1[i]*3+2]) + mat[5]*sx*grid[n1[i]*3] + mat[6]*sy*grid[n1[i]*3+1] + mat[7]*sz*grid[n1[i]*3+2]
        crij_x = (sz*grid[n1[i]*3+1] - sy*grid[n1[i]*3+2]) * dmi[i*3]
        crij_y = (sx*grid[n1[i]*3+2] - sz*grid[n1[i]*3]) * dmi[i*3+1]
        crij_z = (sy*grid[n1[i]*3] - sx*grid[n1[i]*3+1]) * dmi[i*3+2]
        H += (crij_x + crij_y + crij_z) * d_plane
    
    for i in range(6):
        H += mat[2]*(sx*grid[n2[i]*3] + sy*grid[n2[i]*3+1] + sz*grid[n2[i]*3+2]) + mat[8]*sx*grid[n2[i]*3] + mat[9]*sy*grid[n2[i]*3+1] + mat[10]*sz*grid[n2[i]*3+2]
    
    for i in range(6):
        H += mat[3]*(sx*grid[n3[i]*3] + sy*grid[n3[i]*3+1] + sz*grid[n3[i]*3+2]) + mat[11]*sx*grid[n3[i]*3] + mat[12]*sy*grid[n3[i]*3+1] + mat[13]*sz*grid[n3[i]*3+2]
    
    for i in range(12):
        H += mat[4]*(sx*grid[n4[i]*3] + sy*grid[n4[i]*3+1] + sz*grid[n4[i]*3+2]) + mat[14]*sx*grid[n4[i]*3] + mat[15]*sy*grid[n4[i]*3+1] + mat[16]*sz*grid[n4[i]*3+2]
        
    H += mat[17]*sx*sx + mat[18]*sy*sy + mat[19]*sz*sz
    H += B*sz
    return -H

def H_6_6_6_12_DM0(mat, grid, pt, sx, sy, sz, B, size):
    H = 0.0
    n1 = n1_6_6_6_12(pt, size)
    n2 = n2_6_6_6_12(pt, size)
    n3 = n3_6_6_6_12(pt, size)
    n4 = n4_6_6_6_12(pt, size)
    
    for i in range(6):
        H += mat[1]*(sx*grid[n1[i]*3] + sy*grid[n1[i]*3+1] + sz*grid[n1[i]*3+2]) + mat[5]*sx*grid[n1[i]*3] + mat[6]*sy*grid[n1[i]*3+1] + mat[7]*sz*grid[n1[i]*3+2]
    
    for i in range(6):
        H += mat[2]*(sx*grid[n2[i]*3] + sy*grid[n2[i]*3+1] + sz*grid[n2[i]*3+2]) + mat[8]*sx*grid[n2[i]*3] + mat[9]*sy*grid[n2[i]*3+1] + mat[10]*sz*grid[n2[i]*3+2]
    
    for i in range(6):
        H += mat[3]*(sx*grid[n3[i]*3] + sy*grid[n3[i]*3+1] + sz*grid[n3[i]*3+2]) + mat[11]*sx*grid[n3[i]*3] + mat[12]*sy*grid[n3[i]*3+1] + mat[13]*sz*grid[n3[i]*3+2]
    
    for i in range(12):
        H += mat[4]*(sx*grid[n4[i]*3] + sy*grid[n4[i]*3+1] + sz*grid[n4[i]*3+2]) + mat[14]*sx*grid[n4[i]*3] + mat[15]*sy*grid[n4[i]*3+1] + mat[16]*sz*grid[n4[i]*3+2]
        
    H += mat[17]*sx*sx + mat[18]*sy*sy + mat[19]*sz*sz
    H += B*sz
    return -H

def H_4_4_4_8_DM1(mat, grid, pt, sx, sy, sz, dmi, B, size):
    H = 0.0
    d_plane = mat[25]
    n1 = n1_4_4_4_8(pt, size)
    n2 = n2_4_4_4_8(pt, size)
    n3 = n3_4_4_4_8(pt, size)
    n4 = n4_4_4_4_8(pt, size)
    
    for i in range(4):
        H += mat[1]*(sx*grid[n1[i]*3] + sy*grid[n1[i]*3+1] + sz*grid[n1[i]*3+2]) + mat[5]*sx*grid[n1[i]*3] + mat[6]*sy*grid[n1[i]*3+1] + mat[7]*sz*grid[n1[i]*3+2]
        crij_x = (sz*grid[n1[i]*3+1] - sy*grid[n1[i]*3+2]) * dmi[i*3]
        crij_y = (sx*grid[n1[i]*3+2] - sz*grid[n1[i]*3]) * dmi[i*3+1]
        crij_z = (sy*grid[n1[i]*3] - sx*grid[n1[i]*3+1]) * dmi[i*3+2]
        H += (crij_x + crij_y + crij_z) * d_plane
    
    for i in range(4):
        H += mat[2]*(sx*grid[n2[i]*3] + sy*grid[n2[i]*3+1] + sz*grid[n2[i]*3+2]) + mat[8]*sx*grid[n2[i]*3] + mat[9]*sy*grid[n2[i]*3+1] + mat[10]*sz*grid[n2[i]*3+2]
    
    for i in range(4):
        H += mat[3]*(sx*grid[n3[i]*3] + sy*grid[n3[i]*3+1] + sz*grid[n3[i]*3+2]) + mat[11]*sx*grid[n3[i]*3] + mat[12]*sy*grid[n3[i]*3+1] + mat[13]*sz*grid[n3[i]*3+2]
    
    for i in range(8):
        H += mat[4]*(sx*grid[n4[i]*3] + sy*grid[n4[i]*3+1] + sz*grid[n4[i]*3+2]) + mat[14]*sx*grid[n4[i]*3] + mat[15]*sy*grid[n4[i]*3+1] + mat[16]*sz*grid[n4[i]*3+2]
    
    H += mat[17]*sx*sx + mat[18]*sy*sy + mat[19]*sz*sz
    H += B*sz
    return -H

def H_4_4_4_8_DM0(mat, grid, pt, sx, sy, sz, B, size):
    H = 0.0
    n1 = n1_4_4_4_8(pt, size)
    n2 = n2_4_4_4_8(pt, size)
    n3 = n3_4_4_4_8(pt, size)
    n4 = n4_4_4_4_8(pt, size)
    
    for i in range(4):
        H += mat[1]*(sx*grid[n1[i]*3] + sy*grid[n1[i]*3+1] + sz*grid[n1[i]*3+2]) + mat[5]*sx*grid[n1[i]*3] + mat[6]*sy*grid[n1[i]*3+1] + mat[7]*sz*grid[n1[i]*3+2]
    
    for i in range(4):
        H += mat[2]*(sx*grid[n2[i]*3] + sy*grid[n2[i]*3+1] + sz*grid[n2[i]*3+2]) + mat[8]*sx*grid[n2[i]*3] + mat[9]*sy*grid[n2[i]*3+1] + mat[10]*sz*grid[n2[i]*3+2]
    
    for i in range(4):
        H += mat[3]*(sx*grid[n3[i]*3] + sy*grid[n3[i]*3+1] + sz*grid[n3[i]*3+2]) + mat[11]*sx*grid[n3[i]*3] + mat[12]*sy*grid[n3[i]*3+1] + mat[13]*sz*grid[n3[i]*3+2]
    
    for i in range(8):
        H += mat[4]*(sx*grid[n4[i]*3] + sy*grid[n4[i]*3+1] + sz*grid[n4[i]*3+2]) + mat[14]*sx*grid[n4[i]*3] + mat[15]*sy*grid[n4[i]*3+1] + mat[16]*sz*grid[n4[i]*3+2]
    
    H += mat[17]*sx*sx + mat[18]*sy*sy + mat[19]*sz*sz
    H += B*sz
    return -H

def H_3_6_3_6_DM1(mat, grid, pt, sx, sy, sz, dmi, B, size):
    H = 0.0
    d_plane = mat[25]
    n1 = n1_3_6_3_6(pt, size)
    n2 = n2_3_6_3_6(pt, size)
    n3 = n3_3_6_3_6(pt, size)
    n4 = n4_3_6_3_6(pt, size)
    
    for i in range(3):
        H += mat[1]*(sx*grid[n1[i]*3] + sy*grid[n1[i]*3+1] + sz*grid[n1[i]*3+2]) + mat[5]*sx*grid[n1[i]*3] + mat[6]*sy*grid[n1[i]*3+1] + mat[7]*sz*grid[n1[i]*3+2]
        crij_x = (sz*grid[n1[i]*3+1] - sy*grid[n1[i]*3+2]) * dmi[i*3]
        crij_y = (sx*grid[n1[i]*3+2] - sz*grid[n1[i]*3]) * dmi[i*3+1]
        crij_z = (sy*grid[n1[i]*3] - sx*grid[n1[i]*3+1]) * dmi[i*3+2]
        H += (crij_x + crij_y + crij_z) * d_plane
    
    for i in range(3):
        H += mat[2]*(sx*grid[n2[i]*3] + sy*grid[n2[i]*3+1] + sz*grid[n2[i]*3+2]) + mat[8]*sx*grid[n2[i]*3] + mat[9]*sy*grid[n2[i]*3+1] + mat[10]*sz*grid[n2[i]*3+2]
    
    for i in range(3):
        H += mat[3]*(sx*grid[n3[i]*3] + sy*grid[n3[i]*3+1] + sz*grid[n3[i]*3+2]) + mat[11]*sx*grid[n3[i]*3] + mat[12]*sy*grid[n3[i]*3+1] + mat[13]*sz*grid[n3[i]*3+2]
    
    for i in range(6):
        H += mat[4]*(sx*grid[n4[i]*3] + sy*grid[n4[i]*3+1] + sz*grid[n4[i]*3+2]) + mat[14]*sx*grid[n4[i]*3] + mat[15]*sy*grid[n4[i]*3+1] + mat[16]*sz*grid[n4[i]*3+2]
    
    H += mat[17]*sx*sx + mat[18]*sy*sy + mat[19]*sz*sz
    H += B*sz
    return -H

def H_3_6_3_6_DM0(mat, grid, pt, sx, sy, sz, B, size):
    H = 0.0
    n1 = n1_3_6_3_6(pt, size)
    n2 = n2_3_6_3_6(pt, size)
    n3 = n3_3_6_3_6(pt, size)
    n4 = n4_3_6_3_6(pt, size)
    
    for i in range(3):
        H += mat[1]*(sx*grid[n1[i]*3] + sy*grid[n1[i]*3+1] + sz*grid[n1[i]*3+2]) + mat[5]*sx*grid[n1[i]*3] + mat[6]*sy*grid[n1[i]*3+1] + mat[7]*sz*grid[n1[i]*3+2]
    
    for i in range(3):
        H += mat[2]*(sx*grid[n2[i]*3] + sy*grid[n2[i]*3+1] + sz*grid[n2[i]*3+2]) + mat[8]*sx*grid[n2[i]*3] + mat[9]*sy*grid[n2[i]*3+1] + mat[10]*sz*grid[n2[i]*3+2]
    
    for i in range(3):
        H += mat[3]*(sx*grid[n3[i]*3] + sy*grid[n3[i]*3+1] + sz*grid[n3[i]*3+2]) + mat[11]*sx*grid[n3[i]*3] + mat[12]*sy*grid[n3[i]*3+1] + mat[13]*sz*grid[n3[i]*3+2]
    
    for i in range(6):
        H += mat[4]*(sx*grid[n4[i]*3] + sy*grid[n4[i]*3+1] + sz*grid[n4[i]*3+2]) + mat[14]*sx*grid[n4[i]*3] + mat[15]*sy*grid[n4[i]*3+1] + mat[16]*sz*grid[n4[i]*3+2]
    
    H += mat[17]*sx*sx + mat[18]*sy*sy + mat[19]*sz*sz
    H += B*sz
    return -H

def H_2_2_4_2_DM0(mat, grid, pt, sx, sy, sz, dmi, size):
    H = 0.0
    n1 = n1_2_2_4_2(pt, size)
    n2 = n2_2_2_4_2(pt, size)
    n3 = n3_2_2_4_2(pt, size)
    n4 = n4_2_2_4_2(pt, size)
    
    for i in range(2):
        H += mat[1]*(sx*grid[n1[i]*3] + sy*grid[n1[i]*3+1] + sz*grid[n1[i]*3+2]) + mat[5]*sx*grid[n1[i]*3] + mat[6]*sy*grid[n1[i]*3+1] + mat[7]*sz*grid[n1[i]*3+2]
    
    for i in range(2):
        H += mat[2]*(sx*grid[n2[i]*3] + sy*grid[n2[i]*3+1] + sz*grid[n2[i]*3+2]) + mat[8]*sx*grid[n2[i]*3] + mat[9]*sy*grid[n2[i]*3+1] + mat[10]*sz*grid[n2[i]*3+2]
    
    for i in range(4):
        H += mat[3]*(sx*grid[n3[i]*3] + sy*grid[n3[i]*3+1] + sz*grid[n3[i]*3+2]) + mat[11]*sx*grid[n3[i]*3] + mat[12]*sy*grid[n3[i]*3+1] + mat[13]*sz*grid[n3[i]*3+2]
    
    for i in range(2):
        H += mat[4]*(sx*grid[n4[i]*3] + sy*grid[n4[i]*3+1] + sz*grid[n4[i]*3+2]) + mat[14]*sx*grid[n4[i]*3] + mat[15]*sy*grid[n4[i]*3+1] + mat[16]*sz*grid[n4[i]*3+2]
        
    H += mat[17]*sx*sx + mat[18]*sy*sy + mat[19]*sz*sz
    H += B*sz
    return -H

def H_2_4_2_4_DM0(mat, grid, pt, sx, sy, sz, dmi, size):
    H = 0.0
    n1 = n1_2_4_2_4(pt, size)
    n2 = n2_2_4_2_4(pt, size)
    n3 = n3_2_4_2_4(pt, size)
    n4 = n4_2_4_2_4(pt, size)
    
    for i in range(2):
        H += mat[1]*(sx*grid[n1[i]*3] + sy*grid[n1[i]*3+1] + sz*grid[n1[i]*3+2]) + mat[5]*sx*grid[n1[i]*3] + mat[6]*sy*grid[n1[i]*3+1] + mat[7]*sz*grid[n1[i]*3+2]
    
    for i in range(4):
        H += mat[2]*(sx*grid[n2[i]*3] + sy*grid[n2[i]*3+1] + sz*grid[n2[i]*3+2]) + mat[8]*sx*grid[n2[i]*3] + mat[9]*sy*grid[n2[i]*3+1] + mat[10]*sz*grid[n2[i]*3+2]
    
    for i in range(2):
        H += mat[3]*(sx*grid[n3[i]*3] + sy*grid[n3[i]*3+1] + sz*grid[n3[i]*3+2]) + mat[11]*sx*grid[n3[i]*3] + mat[12]*sy*grid[n3[i]*3+1] + mat[13]*sz*grid[n3[i]*3+2]
    
    for i in range(4):
        H += mat[4]*(sx*grid[n4[i]*3] + sy*grid[n4[i]*3+1] + sz*grid[n4[i]*3+2]) + mat[14]*sx*grid[n4[i]*3] + mat[15]*sy*grid[n4[i]*3+1] + mat[16]*sz*grid[n4[i]*3+2]
        
    H += mat[17]*sx*sx + mat[18]*sy*sy + mat[19]*sz*sz
    H += B*sz
    return -H

def Serial_MC_6_6_6_12_DM1(mat, grid, size, dmi, T, BJ):
    n_rn = cp.random.randint(0, size*size, size=size*size).get()
    u_rn = cp.random.uniform(size=size*size).get()
    v_rn = cp.random.uniform(size=size*size).get()
    r_rn = cp.random.uniform(size=size*size).get()
    for j in range(size*size):
        pt = n_rn[j]
        H0 = H_6_6_6_12_DM1(mat, grid, pt, grid[pt*3], grid[pt*3+1], grid[pt*3+2], dmi, BJ, size)
        theta, phi = 2.0*np.pi*u_rn[j], np.arccos(2.0*v_rn[j]-1.0)
        sx, sy, sz = np.sin(phi)*np.cos(theta), np.sin(phi)*np.sin(theta), np.cos(phi)
        H1 = H_6_6_6_12_DM1(mat, grid, pt, sx, sy, sz, dmi, BJ, size)
        if H1 < H0:
            grid[pt*3], grid[pt*3+1], grid[pt*3+2] = sx, sy, sz
        else:
            if r_rn[j] < np.exp(-(H1-H0)*T):
                grid[pt*3], grid[pt*3+1], grid[pt*3+2] = sx, sy, sz

def Serial_MC_6_6_6_12_DM0(mat, grid, size, T, BJ, spin):
    n_rn = cp.random.randint(0, size*size, size=size*size).get()
    u_rn = cp.random.uniform(size=size*size).get()
    v_rn = cp.random.uniform(size=size*size).get()
    r_rn = cp.random.uniform(size=size*size).get()
    for j in range(size*size):
        pt = n_rn[j]
        H0 = H_6_6_6_12_DM0(mat, grid, pt, grid[pt*3], grid[pt*3+1], grid[pt*3+2], BJ, size)
        theta, phi = 2.0*np.pi*u_rn[j], np.arccos(2.0*v_rn[j]-1.0)
        sx, sy, sz = np.sin(phi)*np.cos(theta)*spin, np.sin(phi)*np.sin(theta)*spin, np.cos(phi)*spin
        H1 = H_6_6_6_12_DM0(mat, grid, pt, sx, sy, sz, BJ, size)
        if H1 < H0:
            grid[pt*3], grid[pt*3+1], grid[pt*3+2] = sx, sy, sz
        else:
            if r_rn[j] < np.exp(-(H1-H0)*T):
                grid[pt*3], grid[pt*3+1], grid[pt*3+2] = sx, sy, sz

def Serial_MC_3_6_3_6_DM0(mat, grid, size, T, BJ, spin):
    n_rn = cp.random.randint(0, size*size, size=size*size).get()
    u_rn = cp.random.uniform(size=size*size).get()
    v_rn = cp.random.uniform(size=size*size).get()
    r_rn = cp.random.uniform(size=size*size).get()
    for j in range(size*size):
        pt = n_rn[j]
        H0 = H_3_6_3_6_DM0(mat, grid, pt, grid[pt*3], grid[pt*3+1], grid[pt*3+2], BJ, size)
        theta, phi = 2.0*np.pi*u_rn[j], np.arccos(2.0*v_rn[j]-1.0)
        sx, sy, sz = np.sin(phi)*np.cos(theta)*spin, np.sin(phi)*np.sin(theta)*spin, np.cos(phi)*spin
        H1 = H_3_6_3_6_DM0(mat, grid, pt, sx, sy, sz, BJ, size)
        if H1 < H0:
            grid[pt*3], grid[pt*3+1], grid[pt*3+2] = sx, sy, sz
        else:
            if r_rn[j] < np.exp(-(H1-H0)*T):
                grid[pt*3], grid[pt*3+1], grid[pt*3+2] = sx, sy, sz

class serial_MonteCarlo:
    def __init__(self, config):
        with open(config, "r") as f:
            CONFIG = json.load(f)
                # FLAGS
        self.Single_MAT_Flag = CONFIG["Single_Mat_Flag"]
        self.Static_T_Flag = CONFIG["Static_T_Flag"]
        self.Temps = [np.float32(item) for item in CONFIG["Temps"]]
        self.FM_Flag = CONFIG["FM_Flag"]
        self.DMI_Flag = CONFIG["DMI_Flag"]
        self.TC_Flag = CONFIG["TC_Flag"]
        
        # CONSTANTS AND PATHS
        self.Material = CONFIG["Material"]
        self.Multiple_Materials = CONFIG["Multiple_Materials"]
        self.Input_Folder = "inputs/"
        self.Output_Folder = "outputs/"
        self.B_C = np.float32(CONFIG["B"])
        self.size = CONFIG["SIZE"]
        self.Box = CONFIG["Box"]
        self.Blocks = CONFIG["Blocks"]
        self.Threads = CONFIG["Threads"]
        self.stability_runs = CONFIG["stability_runs"]
        self.calculation_runs = CONFIG["calculation_runs"]
        self.Cmpl = CONFIG["Cmpl_Flag"]
        self.S_Wrap = CONFIG["stability_wrap"]
        self.C_Wrap = CONFIG["calculation_wrap"]
        self.Prefix = CONFIG["Prefix"]
        self.dump_location = "dumps/"
        self.Input_flag = CONFIG["Input_flag"]
        self.Input_File = CONFIG["Input_File"]
        self.dmi_3 = np.load("dmi_3.npy")
        self.dmi_4 = np.load("dmi_4.npy")
        self.dmi_6 = np.load("dmi_6.npy")
        self.MAT_NAME, self.MAT_PARAMS = rm.read_2dmat("../"+self.Input_Folder+"TC_"+self.Material+".csv")
        self.spin = self.MAT_PARAMS[0]
        self.save_direcotry = "../"+self.Output_Folder+self.Prefix+"_"+self.Material+"_"+str(datetime.datetime.now().strftime("%Y_%m_%d_%H_%M_%S")+ "/")
        self.NBS = int(self.MAT_PARAMS[20]), int(self.MAT_PARAMS[21]), int(self.MAT_PARAMS[22]), int(self.MAT_PARAMS[23])
        
        self.metadata = {
            "Material": self.Material,
            "Size": self.size,
            "Box": self.Box,
            "Blocks": self.Blocks,
            "Threads": 2,
            "stability_runs": self.stability_runs,
            "calculation_runs": self.calculation_runs,
            "Cmpl_Flag": self.Cmpl,
            "stability_wrap": self.S_Wrap,
            "calculation_wrap": self.C_Wrap,
            "Prefix": self.Prefix,
            "B": str(self.B_C),
            "spin": str(self.spin),
            "Temps": str(self.Temps),
            "FM_Flag": self.FM_Flag,
            "DMI_Flag": self.DMI_Flag,
            "TC_Flag": self.TC_Flag,
            "Input_flag": self.Input_flag,
            "Input_File": self.Input_File
        }
        metadata_file = json.dumps(self.metadata, indent=4)
        os.mkdir(self.save_direcotry)
        
        self.metadata_file = self.save_direcotry+"/metadata.json"
        with open(self.metadata_file, 'w+') as f:
            f.write(metadata_file)
    
    def display_material(self):
        print(f"\tMaterial: {self.Material}")
        print(f"\tSize: {self.size}")
        print(f"\tSpin: {self.spin}")
        print(f"\tJ1: {self.MAT_PARAMS[1]:.4f} J2: {self.MAT_PARAMS[2]:.4f} J3: {self.MAT_PARAMS[3]:.4f} J4: {self.MAT_PARAMS[4]:.4f}")
        print(f"\tK1x: {self.MAT_PARAMS[5]:.4f} K1y: {self.MAT_PARAMS[6]:.4f} K1z: {self.MAT_PARAMS[7]:.4f}")
        print(f"\tK2x: {self.MAT_PARAMS[8]:.4f} K2y: {self.MAT_PARAMS[9]:.4f} K2z: {self.MAT_PARAMS[10]:.4f}")
        print(f"\tK3x: {self.MAT_PARAMS[11]:.4f} K3y: {self.MAT_PARAMS[12]:.4f} K3z: {self.MAT_PARAMS[13]:.4f}")
        print(f"\tK4x: {self.MAT_PARAMS[14]:.4f} K4y: {self.MAT_PARAMS[15]:.4f} K4z: {self.MAT_PARAMS[16]:.4f}")
        print(f"\tAx: {self.MAT_PARAMS[17]:.4f} Ay: {self.MAT_PARAMS[18]:.4f} Az: {self.MAT_PARAMS[19]:.4f}")
        print(f"\tNBS System: {self.NBS}")
        print(f"\tFile TC/NC: {self.MAT_PARAMS[24]}")
        print(f"\tConfig Temps: {self.Temps}")
    
    def mc_init(self):
        self.size = int(self.size)
        if self.FM_Flag:
            self.grid = np.ones((self.size*self.size*3)).astype(np.float32) * self.spin
        else:
            u, v = np.random.uniform(size=self.size*self.size), np.random.uniform(size=self.size*self.size)
            theta, phi = 2.0*np.pi*u, np.arccos(2.0*v-1.0)
            self.grid = np.zeros((self.size*self.size*3)).astype(np.float32)
            self.grid[0::3] = np.sin(phi)*np.cos(theta)
            self.grid[1::3] = np.sin(phi)*np.sin(theta)
            self.grid[2::3] = np.cos(phi)
            self.grid *= self.spin
        
        if self.Static_T_Flag:
            self.T = self.Temps
        else:
            self.T = np.linspace(0.01, np.float32(2.0*self.MAT_PARAMS[24]), 11)
        self.T_copy = self.T.copy()
        for i in range(len(self.T)):
            self.T[i] = 1/np.float32(self.T[i]*kb)
        
    def grid_reset(self):
        self.grid = np.zeros((self.size*self.size*3)).astype(np.float32)
        if self.FM_Flag:
            mc.FM_N(self.grid, self.size)
        else:
            mc.AFM_N(self.grid, self.size)
        self.grid *= self.spin
    
    def mc_run_6_6_6_12_dm1(self):
        for i in range(self.stability_runs):
            Serial_MC_6_6_6_12_DM1(self.MAT_PARAMS, self.grid, self.size, self.dmi_6.reshape(18), self.T[0], self.B_C)
            if i%10 == 0:
                print("Stability Run: ", i)
            np.save(self.save_direcotry + "grid_" + str(i), self.grid)
    
    def mc_tc_6_6_6_12_dm0(self):
        M, X = [], []
        for j in range(len(self.T)):
            for i in tqdm.tqdm(range(self.calculation_runs)):
                Serial_MC_6_6_6_12_DM0(self.MAT_PARAMS, self.grid, self.size, self.T[j], self.B_C, self.spin)
            
            np.save(self.save_direcotry + "grid_" + str(j), self.grid)
            m = np.sum(self.grid[2::3])/self.size/self.size
            M.append(m)
            X.append(m*m)
            print(f"Temp: {self.T_copy[j]:.4f} M: {m:.4f}")
            print(f"Temp: {self.T_copy[j]:.4f} X: {m*m:.4f}")
            print("-"*20)
        np.save(self.save_direcotry + "M", M)
        np.save(self.save_direcotry + "X", X)
    
    def mc_tc_3_6_3_6_dm0(self):
        M, X = [], []
        for j in range(len(self.T)):
            for i in tqdm.tqdm(range(self.calculation_runs)):
                Serial_MC_3_6_3_6_DM0(self.MAT_PARAMS, self.grid, self.size, self.T[j], self.B_C, self.spin)
            
            np.save(self.save_direcotry + "grid_" + str(j), self.grid)
            m = np.sum(self.grid[2::3])/self.size/self.size
            M.append(m)
            X.append(m*m)
            print(f"Temp: {self.T_copy[j]:.4f} M: {m:.4f}")
            print(f"Temp: {self.T_copy[j]:.4f} X: {m*m:.4f}")
            print("-"*20)
        np.save(self.save_direcotry + "M", M)
        np.save(self.save_direcotry + "X", X)



import construct as cst
import cudametro.montecarlo as mc
import numpy as np
import matplotlib.pyplot as plt



def corr(g1,g2):
    ac = 0
    corr = 0.0
    for i in range(len(g1)):
        for j in range(3):
            cur = g1[i][j]*g2[i][j]
            if cur < 1.0:
                ac += 1
            corr += cur
    
    return corr/(len(g1)*3), ac/(len(g1)*3)



metric = 2000
test_mc0 = cst.MonteCarlo(config="../configs/CRI3.json")
test_mc0.mc_init()
test_mc0.display_material()
gprev = np.reshape(test_mc0.grid, (test_mc0.size**2, 3))/test_mc0.spin
Mf, Xf, Ef = np.array([]), np.array([]), np.array([])
M, X = np.array([]), np.array([])
steps = int(metric*(test_mc0.size**2/(test_mc0.Blocks*test_mc0.stability_runs)))
unit = int(steps/metric)
test_mc0.sampler()
for t in test_mc0.T:
    test_mc0.grid_reset()
    E, C, Ac = np.array([]), np.array([]), np.array([])
    for i in range(steps):
        for j in range(unit):
            test_mc0.generate_random_numbers(test_mc0.S_Wrap)
            m, x = test_mc0.run_mc_tc_3636(t)
        gnext = np.reshape(test_mc0.grid, (test_mc0.size**2, 3))/test_mc0.spin
        c, ac = corr(gprev, gnext)
        gprev = gnext
        M = np.append(M, m)
        X = np.append(X, x)
        test_mc0.sampler()
        e = np.mean(test_mc0.en_3636(t))
        E = np.append(E, e)
        C = np.append(C, c)
        Ac = np.append(Ac, ac)
    print(f"Correlation avg: {np.mean(C):.3f}")
    print(f"Acceptance avg: {np.mean(Ac):.3f}")
    print(f"Energy/site avg: {np.mean(e):.3f} meV")
    plt.plot(E/test_mc0.size**2, label=f"{t:.2f} K")
    np.save(f"{test_mc0.save_direcotry}/Energy_{test_mc0.Blocks}_{t:.2f}", E)
    plt.close()
    plt.plot(C, label=f"{t:.2f} K")
    plt.title(f"Correlation vs Step evolution at {t:.2f} K")
    plt.savefig(f"{test_mc0.save_direcotry}/Correlation_{test_mc0.Blocks}_{t:.2f}.png")
    np.save(f"{test_mc0.save_direcotry}/Correlation_{test_mc0.Blocks}_{t:.2f}", C)
    plt.close()
    plt.plot(Ac, label=f"{t:.2f} K")
    plt.title(f"Acceptance vs Step evolution at {t:.2f} K")
    plt.savefig(f"{test_mc0.save_direcotry}/Acceptance_{test_mc0.Blocks}_{t:.2f}.png")
    np.save(f"{test_mc0.save_direcotry}/Acceptance_{test_mc0.Blocks}_{t:.2f}", Ac)
    plt.close()
    np.save(f"../DiagramData/Fig1/BL_Correlation_{test_mc0.Blocks}_{t:.2f}", C)
    np.save(f"../DiagramData/Fig1/BL_Acceptance_{test_mc0.Blocks}_{t:.2f}", Ac)
    np.save(f"../DiagramData/Fig1/BL_Energy_{test_mc0.Blocks}_{t:.2f}", E)
    Mf = np.append(Mf, np.mean(M))
    Xf = np.append(Xf, np.mean(X))
    Ef = np.append(Ef, np.mean(E))
    np.save(f"{test_mc0.save_direcotry}/grid_{t}", test_mc0.grid.reshape((3*test_mc0.size**2)))
Xf[0] = 0.0
plt.savefig(f"{test_mc0.save_direcotry}/Energy_{test_mc0.size}.png")
plt.legend(str(test_mc0.T),shadow=True, fancybox=True)
plt.title("Energy vs Step evolution")
plt.close()
np.save(f"{test_mc0.save_direcotry}/M_{test_mc0.size}", Mf)
np.save(f"{test_mc0.save_direcotry}/X_{test_mc0.size}", Xf)

plt.plot(test_mc0.T, Mf/test_mc0.spin, label="M")
plt.plot(test_mc0.T, Xf/Xf.max(), label="X")
plt.plot(test_mc0.T, np.gradient(Ef), label="E")
plt.title(f"M/X/E vs T of {test_mc0.MAT_NAME} at size {test_mc0.size}x{test_mc0.size}")
plt.savefig(f"{test_mc0.save_direcotry}/M_{test_mc0.size}.png")



# This file is used to calculate the Curie temperature of a material using Monte Carlo simulation.
#--------------------------------------------

import construct as cst
import cudametro.montecarlo as mc
import numpy as np
import matplotlib.pyplot as plt

test_mc0 = cst.MonteCarlo(config="../../configs/tc_config.json")
test_mc0.mc_init()
test_mc0.display_material()
M, X = np.array([]), np.array([])
for t in test_mc0.T:
    test_mc0.grid_reset()
    test_mc0.generate_random_numbers(test_mc0.S_Wrap)
    m, x = test_mc0.run_mc_tc_3636(t)
    M = np.append(M, m)
    X = np.append(X, x)
np.save(f"{test_mc0.save_direcotry}/BQM_{test_mc0.size}_{test_mc0.Blocks}", M)
np.save(f"{test_mc0.save_direcotry}/BQX_{test_mc0.size}_{test_mc0.Blocks}", X)

X[0] = 0.0

plt.plot(test_mc0.T, M/test_mc0.spin, label="M")
plt.title(f"Magnetization vs Temperature of {test_mc0.MAT_NAME} at size {test_mc0.size}x{test_mc0.size}")
plt.savefig(f"{test_mc0.save_direcotry}/M_{test_mc0.size}.png")
plt.close()
plt.plot(test_mc0.T, X/X.max(), label="X")
plt.title(f"Susceptibility vs Temperature of {test_mc0.MAT_NAME} at size {test_mc0.size}x{test_mc0.size}")
plt.savefig(f"{test_mc0.save_direcotry}/X_{test_mc0.size}.png")
plt.close()



# Constructors and wrappers for the Monte Carlo simulation CUDA code in 'montecarlo.py'
# Create additional directives and algorithms here to run the simulation, more information in the README.md
#--------------------------------------------

import pycuda
import pycuda.autoinit
import pycuda.driver as drv
from pycuda.curandom import rand as curand 

rg = pycuda.curandom.XORWOWRandomNumberGenerator()

import numpy as np 
from numpy import random as rd
import seaborn as sns 
import matplotlib.pyplot as plt 
import matplotlib.cm as cm
from matplotlib.colors import Normalize

import os 
import sys 
import re
script_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, script_dir)
import cudametro.montecarlo as mc
from tqdm import tqdm

import csv
import json
import datetime

def read_2dmat(filename):
    '''
    Reads the 2D material file from the ../../Inputs folder
    '''
    namelist = []
    params_list = np.array([])
    with open(filename, 'r') as f:
        data = f.readlines()
        for item in data:
            subdata = re.split(r'[,|+]', item)
            subdata[-1] = subdata[-1].strip("\n")
            namelist.append(subdata[0])
            params_list = np.append(params_list, np.float32(subdata[1:]))
    return namelist, np.array(params_list, dtype=np.float32)

class MonteCarlo:
    '''
    Default MC class, use this to instantiate the Simulation object
    '''
    def __init__(self, config, input_folder="../../inputs/", output_folder="outputs/"):
        with open(config, 'r') as f:
            CONFIG = json.load(f)
        # FLAGS
        self.Single_MAT_Flag = CONFIG["Single_Mat_Flag"]
        self.Static_T_Flag = CONFIG["Static_T_Flag"]
        self.Temps = [np.float32(item) for item in CONFIG["Temps"]]
        self.FM_Flag = CONFIG["FM_Flag"]
        self.DMI_Flag = CONFIG["DMI_Flag"]
        self.TC_Flag = CONFIG["TC_Flag"]
        
        # CONSTANTS AND PATHS
        self.Material = CONFIG["Material"]
        self.Multiple_Materials = CONFIG["Multiple_Materials"]
        self.Input_Folder = input_folder
        self.Output_Folder = output_folder
        self.B_C = np.float32(CONFIG["B"])
        self.size = CONFIG["SIZE"]
        self.Box = CONFIG["Box"]
        self.Blocks = CONFIG["Blocks"]
        self.Threads = CONFIG["Threads"]
        self.stability_runs = CONFIG["stability_runs"]
        self.calculation_runs = CONFIG["calculation_runs"]
        self.Cmpl = CONFIG["Cmpl_Flag"]
        self.S_Wrap = CONFIG["stability_wrap"]
        self.C_Wrap = CONFIG["calculation_wrap"]
        self.Prefix = CONFIG["Prefix"]
        self.dump_location = "dumps/"
        self.Input_flag = CONFIG["Input_flag"]
        self.Input_File = CONFIG["Input_File"]
        self.C1 = self.Blocks*self.stability_runs
        self.C2 = self.Blocks*self.calculation_runs
        size_int = np.array([self.size]).astype(np.int32)
        self.b = np.array([self.B_C]).astype(np.float32)
        self.GSIZE = drv.mem_alloc(size_int.nbytes)
        self.B_GPU = drv.mem_alloc(self.b.nbytes)
        drv.memcpy_htod(self.GSIZE, size_int)
        drv.memcpy_htod(self.B_GPU, self.b)
        self.dmi_3 = np.load(script_dir+"/dmi_3.npy")
        self.dmi_4 = np.load(script_dir+"/dmi_4.npy")
        self.dmi_6 = np.load(script_dir+"/dmi_6.npy")
        self.GPU_DMI_3 = drv.mem_alloc(self.dmi_3.nbytes)
        self.GPU_DMI_4 = drv.mem_alloc(self.dmi_4.nbytes)
        self.GPU_DMI_6 = drv.mem_alloc(self.dmi_6.nbytes)
        drv.memcpy_htod(self.GPU_DMI_3, self.dmi_3)
        drv.memcpy_htod(self.GPU_DMI_4, self.dmi_4)
        drv.memcpy_htod(self.GPU_DMI_6, self.dmi_6)
        print("Inputs Folder default path: ", self.Input_Folder)
        self.MAT_NAME, self.MAT_PARAMS = read_2dmat(self.Input_Folder+"TC_"+self.Material+".csv")
        self.spin = self.MAT_PARAMS[0]
        spin_gpu = np.array([self.spin]).astype(np.float32)
        self.SGPU = drv.mem_alloc(spin_gpu.nbytes)
        drv.memcpy_htod(self.SGPU, spin_gpu)
        self.save_directory = "Output_"+self.Prefix+"_"+self.Material+"_"+str(datetime.datetime.now().strftime("%Y_%m_%d_%H_%M_%S"))
        self.NBS = int(self.MAT_PARAMS[20]), int(self.MAT_PARAMS[21]), int(self.MAT_PARAMS[22]), int(self.MAT_PARAMS[23])
        print("Output Folder default path: ", self.save_directory)
        self.metadata = {
            "Material": self.Material,
            "Size": self.size,
            "Box": self.Box,
            "Blocks": self.Blocks,
            "Threads": 2,
            "stability_runs": self.stability_runs,
            "calculation_runs": self.calculation_runs,
            "Cmpl_Flag": self.Cmpl,
            "stability_wrap": self.S_Wrap,
            "calculation_wrap": self.C_Wrap,
            "Prefix": self.Prefix,
            "B": str(self.B_C),
            "spin": str(self.spin),
            "Temps": str(self.Temps),
            "FM_Flag": self.FM_Flag,
            "DMI_Flag": self.DMI_Flag,
            "TC_Flag": self.TC_Flag,
            "Input_flag": self.Input_flag,
            "Input_File": self.Input_File
        }
        metadata_file = json.dumps(self.metadata, indent=4)
        os.mkdir(self.save_directory)
        
        self.metadata_file = self.save_directory+"/metadata.json"
        with open(self.metadata_file, 'w+') as f:
            f.write(metadata_file)
        
    def display_material(self):
        '''
        Print material properties
        '''
        print(f"\tMaterial: {self.Material}")
        print(f"\tSize: {self.size}")
        print(f"\tSpin: {self.spin}")
        print(f"\tJ1: {self.MAT_PARAMS[1]:.4f} J2: {self.MAT_PARAMS[2]:.4f} J3: {self.MAT_PARAMS[3]:.4f} J4: {self.MAT_PARAMS[4]:.4f}")
        print(f"\tK1x: {self.MAT_PARAMS[5]:.4f} K1y: {self.MAT_PARAMS[6]:.4f} K1z: {self.MAT_PARAMS[7]:.4f}")
        print(f"\tK2x: {self.MAT_PARAMS[8]:.4f} K2y: {self.MAT_PARAMS[9]:.4f} K2z: {self.MAT_PARAMS[10]:.4f}")
        print(f"\tK3x: {self.MAT_PARAMS[11]:.4f} K3y: {self.MAT_PARAMS[12]:.4f} K3z: {self.MAT_PARAMS[13]:.4f}")
        print(f"\tK4x: {self.MAT_PARAMS[14]:.4f} K4y: {self.MAT_PARAMS[15]:.4f} K4z: {self.MAT_PARAMS[16]:.4f}")
        print(f"\tAx: {self.MAT_PARAMS[17]:.4f} Ay: {self.MAT_PARAMS[18]:.4f} Az: {self.MAT_PARAMS[19]:.4f}")
        print(f"\tNBS System: {self.NBS}")
        print(f"\tFile TC/NC: {self.MAT_PARAMS[24]}")
        print(f"\tConfig Temps: {self.Temps}")
        
    def generate_random_numbers(self):
        '''
        Generate 4 uniform random number arrays for the simulation and copies them to the GPU
        '''
        self.NLIST = rg.gen_uniform((self.C1), np.float32)
        self.ULIST = rg.gen_uniform((self.C1), np.float32)
        self.VLIST = rg.gen_uniform((self.C1), np.float32)
        self.RLIST = rg.gen_uniform((self.C1), np.float32)

        self.NFULL = pycuda.gpuarray.zeros((self.C1), dtype=np.int32)
        self.S1FULL = pycuda.gpuarray.zeros((self.C1), dtype=np.float32)
        self.S2FULL = pycuda.gpuarray.zeros((self.C1), dtype=np.float32)
        self.S3FULL = pycuda.gpuarray.zeros((self.C1), dtype=np.float32)

        mc.NPREC(self.NLIST, self.NFULL, self.GSIZE, block=(1,1,1), grid=(self.C1,1,1))
        mc.VPREC(self.ULIST, self.VLIST, self.S1FULL, self.S2FULL, self.S3FULL, self.SGPU, block=(1,1,1), grid=(self.C1,1,1))

    def generate_ising_numbers(self):
        '''
        Generate 4 uniform random number arrays for the simulation and copies them to the GPU
        '''
        self.NLIST = rg.gen_uniform((self.C1), np.float32)
        self.ULIST = rg.gen_uniform((self.C1), np.float32)
        self.VLIST = rg.gen_uniform((self.C1), np.float32)
        self.RLIST = rg.gen_uniform((self.C1), np.float32)

        self.NFULL = pycuda.gpuarray.zeros((self.C1), dtype=np.int32)
        self.S1FULL = pycuda.gpuarray.zeros((self.C1), dtype=np.float32)
        self.S2FULL = pycuda.gpuarray.zeros((self.C1), dtype=np.float32)
        self.S3FULL = pycuda.gpuarray.zeros((self.C1), dtype=np.float32)

        mc.NPREC(self.NLIST, self.NFULL, self.GSIZE, block=(1,1,1), grid=(self.C1,1,1))
        mc.ISING(self.ULIST, self.VLIST, self.S1FULL, self.S2FULL, self.S3FULL, self.SGPU, block=(1,1,1), grid=(self.C1,1,1))
    
    def sampler(self):
        self.N_SAMPLE = rg.gen_uniform((self.Blocks), np.float32)
        self.GPU_N_SAMPLE = drv.mem_alloc(self.N_SAMPLE.nbytes)
        mc.NPREC(self.N_SAMPLE, self.GPU_N_SAMPLE, self.GSIZE, block=(1,1,1), grid=(self.Blocks,1,1))

    def mc_init(self):
        '''
        Initialize the simulation
        '''
        self.grid = np.zeros((self.size*self.size*3)).astype(np.float32)
        self.GRID_GPU = drv.mem_alloc(self.grid.nbytes)
        self.TMATRIX = np.zeros((self.Blocks, 4)).astype(np.float32)
        self.GPU_TRANS = drv.mem_alloc(self.TMATRIX.nbytes)
        if self.FM_Flag:
            mc.FM_N(self.grid, self.size)
        else:
            mc.AFM_N(self.grid, self.size)
        self.grid *= self.spin
        self.GPU_MAT = drv.mem_alloc(self.MAT_PARAMS.nbytes)
        
        print(self.GRID_GPU)
        if self.Static_T_Flag:
            self.T = self.Temps
        else:
            self.T = np.linspace(0.01, np.float32(2.0*self.MAT_PARAMS[24]), 11)
        self.BJ = drv.mem_alloc(self.T[0].nbytes)
        drv.memcpy_htod(self.GPU_MAT, self.MAT_PARAMS)
        drv.memcpy_htod(self.GRID_GPU, self.grid)
    
    def grid_reset(self):
        self.grid = np.zeros((self.size*self.size*3)).astype(np.float32)
        if self.FM_Flag:
            mc.FM_N(self.grid, self.size)
        else:
            mc.AFM_N(self.grid, self.size)
        self.grid *= self.spin
        drv.memcpy_htod(self.GRID_GPU, self.grid)
    '''
    DMI SECTOR
    This section has the MC simulation algorithms for materials that use a DMI term
    In order, this contains the following functions:
    run_mc_dmi_66612: Runs the MC simulation for a 6 6 6 12 crystal structure material
    run_mc_dmi_4448: Runs the MC simulation for a 4 4 4 8 crystal structure material
    run_mc_dmi_3636: Runs the MC simulation for a 3 6 3 6 crystal structure material
    '''
    def run_mc_dmi_66612(self, T):
        beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
        drv.memcpy_htod(self.BJ,beta[0])
        for j in range(self.stability_runs):
            mc.METROPOLIS_MC_DM1_6_6_6_12(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.GPU_DMI_6, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
            mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
        drv.memcpy_dtoh(self.grid, self.GRID_GPU)
        return self.grid

    def run_mc_dmi_4448(self, T):
        beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
        drv.memcpy_htod(self.BJ,beta[0])
        for j in range(self.stability_runs):
            mc.METROPOLIS_MC_DM1_4_4_4_8(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.GPU_DMI_6, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
            mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
        drv.memcpy_dtoh(self.grid, self.GRID_GPU)
        return self.grid
    
    def run_mc_dmi_3636(self, T):
        beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
        drv.memcpy_htod(self.BJ,beta[0])
        for j in range(self.stability_runs):
            mc.METROPOLIS_MC_DM1_3_6_3_6(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.GPU_DMI_3, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
            mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
        drv.memcpy_dtoh(self.grid, self.GRID_GPU)
        return self.grid
    
    
    # DMI SECTOR END
    
    # TC SECTOR
    # This section has the MC simulation algorithms for finding the TC of a material
    
    def run_mc_tc_4448(self, T):
        '''
        Run the MC simulation for a 4 4 4 8 crystal structure material
        '''
        Mt, Xt = 0.0, 0.0
        ct = 0
        M, X = np.zeros(self.S_Wrap), np.zeros(self.S_Wrap)
        for i in tqdm(range(self.S_Wrap), desc=f"Stabilizing at {T:.3f}", colour="blue"):
            mag_fluc =  np.zeros(self.stability_runs)
            beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
            drv.memcpy_htod(self.BJ, beta)
            for j in range(self.stability_runs):
                mc.METROPOLIS_MC_DM0_4_4_4_8(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
                mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
            drv.memcpy_dtoh(self.grid, self.GRID_GPU)
            self.grid = self.grid.reshape((self.size, self.size, 3))
            magx, magy, magz = self.grid[:,:,0], self.grid[:,:,1], self.grid[:,:,2]
            mag = np.array([np.sum(magx) , np.sum(magy) , np.sum(magz)])/(self.size**2)
            M[i] = np.abs(np.linalg.norm(mag))
        Mt, Xt = np.mean(M), np.std(M)/T
        print(f"Mean Magnetization at {T:.3f} = {Mt:.3f}")
        print(f"Mean Susceptibility at {T:.3f} = {Xt:.3f}")
        
        return Mt, Xt
    
    def run_mc_tc_66612(self, T):
        '''
        Run the MC simulation for a 6 6 6 12 crystal structure material
        '''
        Mt, Xt = 0.0, 0.0
        ct = 0
        M, X = np.zeros(self.S_Wrap), np.zeros(self.S_Wrap)
        for i in tqdm(range(self.S_Wrap), desc=f"Stabilizing at {T:.3f}", colour="blue"):
            mag_fluc =  np.zeros(self.stability_runs)
            beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
            drv.memcpy_htod(self.BJ, beta)
            for j in range(self.stability_runs):
                mc.METROPOLIS_MC_DM0_6_6_6_12(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
                mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
            drv.memcpy_dtoh(self.grid, self.GRID_GPU)
            self.grid = self.grid.reshape((self.size, self.size, 3))
            magx, magy, magz = self.grid[:,:,0], self.grid[:,:,1], self.grid[:,:,2]
            mag = np.array([np.sum(magx) , np.sum(magy) , np.sum(magz)])/(self.size**2)
            M[i] = np.abs(np.linalg.norm(mag))
        Mt, Xt = np.mean(M), np.std(M)/T
        print(f"Mean Magnetization at {T:.3f} = {Mt:.3f}")
        print(f"Mean Susceptibility at {T:.3f} = {Xt:.3f}")
        
        return Mt, Xt
    
    def run_mc_tc_3636(self, T):
        '''
        Run the MC simulation for a 3 6 3 6 crystal structure material
        '''
        Mt, Xt = 0.0, 0.0
        ct = 0
        M = np.zeros(self.S_Wrap)
        for i in tqdm(range(self.S_Wrap), desc=f"Stabilizing at {T:.3f}", colour="blue"):
            mag_fluc =  np.zeros(self.stability_runs)
            beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
            drv.memcpy_htod(self.BJ, beta)
            for j in range(self.stability_runs):
                mc.METROPOLIS_MC_DM0_3_6_3_6(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
                mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
            drv.memcpy_dtoh(self.grid, self.GRID_GPU)
            self.grid = self.grid.reshape((self.size, self.size, 3))
            magx, magy, magz = self.grid[:,:,0], self.grid[:,:,1], self.grid[:,:,2]
            mag = np.array([np.sum(magx) , np.sum(magy) , np.sum(magz)])/(self.size**2)
            M[i] = np.abs(np.linalg.norm(mag))
        Mt, Xt = np.mean(M), np.std(M)/T
        print(f"Mean Magnetization at {T:.3f} = {Mt:.3f}")
        print(f"Mean Susceptibility at {T:.3f} = {Xt:.3f}")
        
        return Mt, Xt
    
    '''
    # Deprecated function
    def run_mc_tc_3636_2(self, T):
        Mt, Xt = 0.0, 0.0
        ct = 0
        M = np.zeros(self.S_Wrap)
        for i in tqdm(range(self.S_Wrap), desc=f"Stabilizing at {T:.3f}", colour="blue"):
            mag_fluc =  np.zeros(self.stability_runs)
            beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
            drv.memcpy_htod(self.BJ, beta)
            for j in range(self.stability_runs):
                mc.METROPOLIS_MC_DM2_3_6_3_6(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
                mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
            drv.memcpy_dtoh(self.grid, self.GRID_GPU)
            self.grid = self.grid.reshape((self.size, self.size, 3))
            magx, magy, magz = self.grid[:,:,0], self.grid[:,:,1], self.grid[:,:,2]
            mag = np.array([np.sum(magx) , np.sum(magy) , np.sum(magz)])/(self.size**2)
            M[i] = np.abs(np.linalg.norm(mag))
        Mt, Xt = np.mean(M), np.std(M)/T
        print(f"Mean Magnetization at {T:.3f} = {Mt:.3f}")
        print(f"Mean Susceptibility at {T:.3f} = {Xt:.3f}")
        
        return Mt, Xt
    '''

    def run_mc_tc_2242(self, T):
        '''
        Run the MC simulation for a 2 2 4 2 crystal structure material
        '''
        Mt, Xt = 0.0, 0.0
        ct = 0
        M = np.zeros(self.S_Wrap)
        for i in tqdm(range(self.S_Wrap), desc=f"Stabilizing at {T:.3f}", colour="blue"):
            mag_fluc =  np.zeros(self.stability_runs)
            beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
            drv.memcpy_htod(self.BJ, beta)
            for j in range(self.stability_runs):
                mc.METROPOLIS_MC_DM0_2_2_4_2(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
                mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
            drv.memcpy_dtoh(self.grid, self.GRID_GPU)
            self.grid = self.grid.reshape((self.size, self.size, 3))
            magx, magy, magz = self.grid[:,:,0], self.grid[:,:,1], self.grid[:,:,2]
            mag = np.array([np.sum(magx) , np.sum(magy) , np.sum(magz)])/(self.size**2)
            M[i] = np.abs(np.linalg.norm(mag))
        Mt, Xt = np.mean(M), np.std(M)/T
        print(f"Mean Magnetization at {T:.3f} = {Mt:.3f}")
        print(f"Mean Susceptibility at {T:.3f} = {Xt:.3f}")
        
        return Mt, Xt
    
    def run_mc_tc_2424(self, T):
        '''
        Run the MC simulation for a 2 4 2 4 crystal structure material
        '''
        Mt, Xt = 0.0, 0.0
        ct = 0
        M = np.zeros(self.S_Wrap)
        for i in tqdm(range(self.S_Wrap), desc=f"Stabilizing at {T:.3f}", colour="blue"):
            mag_fluc =  np.zeros(self.stability_runs)
            beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
            drv.memcpy_htod(self.BJ, beta)
            for j in range(self.stability_runs):
                mc.METROPOLIS_MC_DM0_2_4_2_4(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
                mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
            drv.memcpy_dtoh(self.grid, self.GRID_GPU)
            self.grid = self.grid.reshape((self.size, self.size, 3))
            magx, magy, magz = self.grid[:,:,0], self.grid[:,:,1], self.grid[:,:,2]
            mag = np.array([np.sum(magx) , np.sum(magy) , np.sum(magz)])/(self.size**2)
            M[i] = np.abs(np.linalg.norm(mag))
        Mt, Xt = np.mean(M), np.std(M)/T
        print(f"Mean Magnetization at {T:.3f} = {Mt:.3f}")
        print(f"Mean Susceptibility at {T:.3f} = {Xt:.3f}")
        
        return Mt, Xt

    # TC SECTOR END
    
    # TC EN SECTOR
    # This section has the MC simulation algorithms for finding the Energy/atom of a material while simulating.
    # In order, this contains the following functions:

    def run_mc_tc_en_3636(self, T):
        '''
        Run the MC simulation for a 3 6 3 6 crystal structure material while calculating the Energy/atom
        '''
        Mt, Xt = 0.0, 0.0
        ct = 0
        M = np.zeros(self.S_Wrap)
        E = np.zeros(self.S_Wrap)
        Et = np.zeros(self.Blocks).astype(np.float32)
        GPU_ET = drv.mem_alloc(Et.nbytes)

        for i in tqdm(range(self.S_Wrap), desc=f"Stabilizing at {T:.3f}", colour="blue"):
            mag_fluc =  np.zeros(self.stability_runs)
            beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
            drv.memcpy_htod(self.BJ, beta)
            ET_S = np.zeros(self.stability_runs)
            for j in range(self.stability_runs):
                mc.METROPOLIS_MC_DM0_3_6_3_6(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
                mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
                self.sampler()
                mc.EN_CALC_3_6_3_6(self.GPU_MAT, self.GRID_GPU, self.GPU_N_SAMPLE, GPU_ET, self.B_GPU, self.GSIZE, block=(1,1,1), grid=(self.size*self.size,1,1))
                drv.memcpy_dtoh(Et, GPU_ET)
                ET_S[j] = np.mean(Et)
            drv.memcpy_dtoh(self.grid, self.GRID_GPU)
            self.grid = self.grid.reshape((self.size, self.size, 3))
            magx, magy, magz = self.grid[:,:,0], self.grid[:,:,1], self.grid[:,:,2]
            mag = np.array([np.sum(magx) , np.sum(magy) , np.sum(magz)])/(self.size**2)
            M[i] = np.abs(np.linalg.norm(mag))
            E[i] = np.mean(ET_S)
        Mt, Xt, Et = np.mean(M), np.std(M)/T, np.std(E[-10:])/T**2
        np.save(f"{self.save_directory}/En_{T:.3f}", E)
        print(f"Mean Magnetization at {T:.3f} = {Mt:.3f}")
        print(f"Mean Susceptibility at {T:.3f} = {Xt:.3f}")
        print(f"Mean Specific Heat at {T:.3f} = {Et:.3f}")
        return Mt, Xt, Et
    '''
    def run_mc_tc_en_3636_2(self, T):
        Mt, Xt = 0.0, 0.0
        ct = 0
        M = np.zeros(self.S_Wrap)
        E = np.zeros(self.S_Wrap)
        Et = np.zeros(self.Blocks).astype(np.float32)
        GPU_ET = drv.mem_alloc(Et.nbytes)

        for i in tqdm(range(self.S_Wrap), desc=f"Stabilizing at {T:.3f}", colour="blue"):
            mag_fluc =  np.zeros(self.stability_runs)
            beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
            drv.memcpy_htod(self.BJ, beta)
            ET_S = np.zeros(self.stability_runs)
            for j in range(self.stability_runs):
                mc.METROPOLIS_MC_DM1_3_6_3_6(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
                mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
                self.sampler()
                mc.EN_CALC_3_6_3_6_2(self.GPU_MAT, self.GRID_GPU, self.GPU_N_SAMPLE, GPU_ET, self.B_GPU, self.GSIZE, block=(1,1,1), grid=(self.size*self.size,1,1))
                drv.memcpy_dtoh(Et, GPU_ET)
                ET_S[j] = np.mean(Et)
            drv.memcpy_dtoh(self.grid, self.GRID_GPU)
            self.grid = self.grid.reshape((self.size, self.size, 3))
            magx, magy, magz = self.grid[:,:,0], self.grid[:,:,1], self.grid[:,:,2]
            mag = np.array([np.sum(magx) , np.sum(magy) , np.sum(magz)])/(self.size**2)
            M[i] = np.abs(np.linalg.norm(mag))
            E[i] = np.mean(ET_S)
        Mt, Xt, Et = np.mean(M), np.std(M)/T, np.std(E[-10:])/T**2
        np.save(f"{self.save_directory}/En_{T:.3f}", E)
        print(f"Mean Magnetization at {T:.3f} = {Mt:.3f}")
        print(f"Mean Susceptibility at {T:.3f} = {Xt:.3f}")
        print(f"Mean Specific Heat at {T:.3f} = {Et:.3f}")
        return Mt, Xt, Et
    '''
    def en_3636(self, T):
        '''
        Calculate the Energy/atom for a 3 6 3 6 crystal structure material
        '''
        Et = np.zeros(self.Blocks).astype(np.float32)
        GPU_ET = drv.mem_alloc(Et.nbytes)
        mc.EN_CALC_3_6_3_6(self.GPU_MAT, self.GRID_GPU, self.B_GPU, self.GPU_N_SAMPLE, self.GSIZE, GPU_ET, block=(1,1,1), grid=(self.Blocks,1,1))
        drv.memcpy_dtoh(Et, GPU_ET)

        return Et
    
    '''
    def en_3636_2(self, T):
        Et = np.zeros(self.Blocks).astype(np.float32)
        GPU_ET = drv.mem_alloc(Et.nbytes)
        mc.EN_CALC_3_6_3_6_2(self.GPU_MAT, self.GRID_GPU, self.B_GPU, self.GPU_N_SAMPLE, self.GSIZE, GPU_ET, block=(1,1,1), grid=(self.Blocks,1,1))
        drv.memcpy_dtoh(Et, GPU_ET)
        
        return Et
    '''
    def run_mc_tc_en_66612(self, T):
        '''
        Calculate the Energy/atom for a 6 6 6 12 crystal structure material
        '''
        Et = np.zeros(self.Blocks).astype(np.float32)
        GPU_ET = drv.mem_alloc(Et.nbytes)
        mc.EN_CALC_6_6_6_12(self.GPU_MAT, self.GRID_GPU, self.B_GPU, self.GPU_N_SAMPLE, self.GSIZE, GPU_ET, block=(1,1,1), grid=(self.Blocks,1,1))
        drv.memcpy_dtoh(Et, GPU_ET)
        
        return Et
        
    # TC EN SECTOR END
    
    # MISC SECTOR
    
    '''
    def run_mc_dmi_36362(self, T):
        beta = np.array([1.0 / (T * 8.6173e-2)],dtype=np.float32)
        drv.memcpy_htod(self.BJ,beta[0])
        for j in range(self.stability_runs):
            mc.METROPOLIS_MC_DM2_3_6_3_6(self.GPU_MAT, self.GRID_GPU, self.BJ, self.NFULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S1FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S2FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.S3FULL[j*self.Blocks:(j+1)*self.Blocks-1], self.RLIST[j*self.Blocks:(j+1)*self.Blocks-1], self.GPU_TRANS, self.B_GPU, self.GSIZE, block=(self.Threads,1,1), grid=(self.Blocks,1,1))
            mc.GRID_COPY(self.GRID_GPU, self.GPU_TRANS, block=(1,1,1), grid=(self.Blocks,1,1))
        drv.memcpy_dtoh(self.grid, self.GRID_GPU)
        return self.grid
    
    def dump_state():
        pass
    '''
    
    def cleanup(self):
        '''
        Clean up the GPU GRID memory
        '''
        self.GRID_GPU.free()

class Analyze():
    def __init__(self, directory, reverse=False, input_folder="../../inputs/"):
        '''
        Initialize the Analyze class, used to analyze the output of the MC simulation
        '''
        self.flist = os.listdir(directory)
        self.flist = [file for file in self.flist if file.endswith(".npy") and file.startswith("grid")]
        self.flist.sort(reverse=reverse)
        self.directory = directory
        if not os.path.exists(self.directory+"/spin"):
            os.mkdir(self.directory+"/spin")
        if not os.path.exists(self.directory+"/quiver"):
            os.mkdir(self.directory+"/quiver")
        print(f"{len(self.flist)} files found.....Analyzing")
        self.metadata = json.load(open(self.directory+"/metadata.json", 'r'))
        self.size = self.metadata["Size"]
        self.spin = np.float32(self.metadata["spin"])
        self.Blocks = self.metadata["Blocks"]
        self.Material = self.metadata["Material"]
        self.Input_Folder = input_folder
        self.MAT_NAME, self.MAT_PARAMS = read_2dmat(self.Input_Folder+"TC_"+self.Material+".csv")
        self.GPU_MAT = drv.mem_alloc(self.MAT_PARAMS.nbytes)
        drv.memcpy_htod(self.GPU_MAT, self.MAT_PARAMS)
        self.B_GPU = drv.mem_alloc(self.MAT_PARAMS[0].nbytes)
        drv.memcpy_htod(self.B_GPU, np.array([self.metadata["B"]]).astype(np.float32))
        self.GSIZE = drv.mem_alloc(np.array([self.size]).astype(np.int32).nbytes)
        drv.memcpy_htod(self.GSIZE, np.array([self.size]).astype(np.int32))
        #DMI_6 = np.load("dmi_6.npy")
        #self.GPU_DMI_6 = drv.mem_alloc(DMI_6.nbytes)
        #drv.memcpy_htod(self.GPU_DMI_6, DMI_6)


    def spin_view(self):
        '''
        Generate a heatmap of the spin configuration at each time step
        '''
        ctr = 0
        for file in self.flist:
            print(f"Processing {file}", end="\r")
            grid = np.load(self.directory+"/"+file)
            shape = grid.shape
            with open(self.directory+"/metadata.json", 'r') as f:
                metadata = json.load(f)
            self.spin = np.float32(metadata["spin"])
            grid = grid.reshape((int(np.sqrt(shape[0]/3)), int(np.sqrt(shape[0]/3)), 3))
            #grid = grid.reshape((64, 64, 3))
            spinx, spiny, spinz = grid[:,:,0], grid[:,:,1], grid[:,:,2]
            figure = plt.figure(dpi=400)
            plt.title("Spin Configuration at T = "+str(ctr))
            ax = figure.add_subplot(131)
            sns.heatmap(spinz, cbar=False, cmap="coolwarm", square=True, xticklabels=False, yticklabels=False, vmin=-self.spin, vmax=self.spin)
            ax.set_xlabel("Z")
            ax = figure.add_subplot(132)
            sns.heatmap(spiny, cbar=False, cmap="coolwarm", square=True, xticklabels=False, yticklabels=False, vmin=-self.spin, vmax=self.spin)
            ax.set_xlabel("Y")
            ax = figure.add_subplot(133)
            sns.heatmap(spinx, cbar=False, cmap="coolwarm", square=True, xticklabels=False, yticklabels=False, vmin=-self.spin, vmax=self.spin)
            ax.set_xlabel("X")
            plt.savefig(self.directory+"/spin/spin_"+str(ctr)+".png")
            plt.close()
            ctr += 1
    
    def quiver_view(self):
        '''
        Generate a quiver plot of the spin configuration at each time step
        '''
        ctr = 0
        for file in self.flist:
            print(f"Processing {file}", end="\r")
            grid = np.load(self.directory+"/"+file)
            shape = grid.shape
            grid = grid.reshape((int(np.sqrt(shape[0]/3)), int(np.sqrt(shape[0]/3)), 3))
            spinx, spiny, spinz = grid[:,:,0], grid[:,:,1], grid[:,:,2]
            x_mesh, y_mesh = np.meshgrid(np.arange(0, int(np.sqrt(shape[0]/3)), 1), np.arange(0, int(np.sqrt(shape[0]/3)), 1))
            figure = plt.figure(figsize=(10,10),dpi=400)
            plt.title("Spin Configuration at T = "+str(ctr))
            ax = figure.add_subplot(111)
            rgba = np.zeros((shape[0],4))
            spinz = np.reshape(spinz, int(shape[0]/3))
            norm = Normalize()
            norm.autoscale(spinz)
            colormap = cm.bwr
            plt.quiver(x_mesh, y_mesh, spinx, spiny, scale=self.spin, scale_units="xy", pivot="mid", color=colormap(norm(spinz)), width=0.01, headwidth=3, headlength=4, headaxislength=3, minlength=0.1, minshaft=1)
            plt.savefig(self.directory+"/quiver/quiver_"+str(ctr)+".png")
            plt.close()
            ctr += 1
        
    def en_66612(self):
        '''
        Visualize the Energy/atom for a 6 6 6 12 crystal structure material
        '''
        ctr = 0
        E_f = np.zeros(len(self.flist))
        for file in self.flist:
            print(f"Processing {file}", end="\r")
            grid = np.load(self.directory+"/"+file)
            shape = grid.shape
            grid = grid.reshape((int(np.sqrt(shape[0]/3)), int(np.sqrt(shape[0]/3)), 3))
            Et = np.zeros(self.Blocks).astype(np.float32)
            self.GRID_GPU = drv.mem_alloc(grid.nbytes)
            drv.memcpy_htod(self.GRID_GPU, grid)
            GPU_ET = drv.mem_alloc(Et.nbytes)
            mc.EN_CALC_6_6_6_12(self.GPU_MAT, self.GRID_GPU, self.B_GPU, self.GSIZE, GPU_ET, self.GPU_DMI_6, block=(1,1,1), grid=(self.Blocks,1,1))
            drv.memcpy_dtoh(Et, GPU_ET)
            print(np.mean(-Et))
            E_f[ctr] = -np.mean(Et)
            ctr += 1
        np.save(self.directory+"/En", E_f)
        plt.plot(E_f)
        plt.savefig(self.directory+"/En.png")
        np.savetxt(self.directory+"/En.txt", E_f)
        


    
    
    


from cudametro import construct as cst
import numpy as np
import matplotlib.pyplot as plt
import tqdm as tqdm


# =============================================================================
# Edit the following lines to change the parameters of the simulation
# =============================================================================
fname = "fig3_configs/fig3.json"          # Configuration file [options fig2a.json, fig2b.json]
in_dir = "input_parameters/"               # Input directory
# =============================================================================

# Set your config file here
test_mc0 = cst.MonteCarlo(config=fname, input_folder=in_dir)

# Initialize the Monte Carlo simulation
test_mc0.mc_init()
test_mc0.display_material()
for i in tqdm.tqdm(range(test_mc0.S_Wrap), desc="Stability Runs", unit="runs", colour="green"):
    test_mc0.generate_random_numbers(test_mc0.stability_runs)
    np.save(f"{test_mc0.save_directory}/grid_{i:04d}", test_mc0.run_mc_dmi_66612(test_mc0.T[0]))

# Save the final state of the simulation 
# To visualize the final state, use the "visualize.py" script with appropriate directory

output_folder_name = test_mc0.save_directory

folder = output_folder_name

viewer = cst.Analyze(folder, reverse=False, input_folder=in_dir)

# Visualize the results
# Spin configuration at the end of the simulation
viewer.spin_view()
# Quiver plot of the spins
viewer.quiver_view()
# The energy of the system
viewer.en_66612()

# Your Diagrams are now saved in the output folder. Use the 'reverse' flag if you want to quickly visualize the end state. 
# You can even delete data points should you wish to speed up your visualizer.
# Note that your energy will also be counted in reverse if you used the reverse flag.


# Benchmarking script for the Monte Carlo simulation

#--------------------------------------------
import construct as cst
import cudametro.montecarlo as mc
import numpy as np
import tqdm as tqdm
import datetime as dt
import pycuda.autoinit
import pycuda.driver as cuda
import sys


tol = 1e-3

(free,total)=cuda.mem_get_info()
print("Global memory occupancy:%f%% free"%(free*100/total))

device=cuda.Device(0)
print(f"Name: {device.name()}")
print(f"Compute Capability: {device.compute_capability()}")
print(f"Total Memory: {device.total_memory()/1024**3:.1f} GB")
print("=====================================")

sizes = [64,128,256,512]
blocks = [64,128,256,512,1024,2048,4096,8192,16384]

print("Benching with approx 90.0% memory usage")

bench_MC = cst.MonteCarlo(config="../configs/bench.json")
base_stability_runs = int(device.total_memory()*0.90/(10000*sys.getsizeof(np.float32())))
print(f"clean base stability runs on single spin update: {base_stability_runs}")


bench_MC.size = 256
bench_MC.mc_init()
Time = np.array([])
print("Begin Bench")
for i in range(len(blocks)):
    if blocks[i] > base_stability_runs:
        break
    bench_MC.Blocks = blocks[i]
    bench_MC.stability_runs = 10000*base_stability_runs//bench_MC.Blocks
    print(f"Blocks: {bench_MC.Blocks}, Stability Runs: {bench_MC.stability_runs}")
    bench_MC.C1 = bench_MC.Blocks*bench_MC.stability_runs
    t_curr = dt.datetime.now()
    for j in range(bench_MC.S_Wrap):
        bench_MC.generate_random_numbers(1)
        bench_MC.run_mc_tc_3636(bench_MC.T[0])
    t_end = dt.datetime.now()
    Time = np.append(Time, (t_end-t_curr).total_seconds())
    print(f"Time taken for {blocks[i]} blocks: {t_end-t_curr}")

np.save(f"{bench_MC.save_direcotry}/Timings", Time)
print("=====================================")
